import pandas as pd

# reading
df_expression = pd.read_table("data_files/GSE41558_series_matrix.reduced.tsv" , index_col=0)
print(df.head())

print( ' *** ' )

# applying a custom function 

def expressionFilter(iterable , minExpr=10 , minSample=5):
    ''' returns True is at least <minSample> have a value of at least <minExpr> in <iterable>'''
    mask = iterable>=minExpr  # creating a mask to check for mininmum expression
    return sum(mask)>=minSample # Return True if at least minSample samples satisfy the filter


# apply along rows
expressionFilterMask =  df_expression.apply(expressionFilter , axis=1) 
print( expressionFilterMask.value_counts() ) # we print the count of the custom mask we just created

print( ' *** ' )

import numpy as np # numpy is a module that contains a lot of useful mathematical functions. \
# Here we import it for the logarithm and average functions

def calculate_mean_log(iterable):
    """returns the average of the logarithm of the values in iterable"""
    return np.average( np.log(iterable) )

# apply along columns
meanLogexpression = df_expression.apply(calculate_mean_log, axis=0)
print( meanLogexpression )

print( '***' )

print( df_expression['Heart_WT_1'].head() )

print( df_expression['Heart_WT_1'].head() / 1000 )

print( ' *** ' )

## performing operation on columns 

df_expression['Heart_WT_1'] + df_expression['Heart_WT_2']

print( df_expression['Heart_WT_1'] + df_expression['Heart_WT_2'] )


print( ' *** ' )

meanExpressionWT = ( df_expression['Heart_WT_1'] + 
                     df_expression['Heart_WT_2'] + 
                     df_expression['Heart_WT_3'] + 
                     df_expression['Heart_WT_4'] ) / 4

print( meanExpressionWT )


print( ' *** ' )

df_expression['Heart_WT_avg'] = meanExpressionWT

print( df_expression.head() )

print( ' *** ' )

df_expression.drop('Heart_WT_avg', axis=1, inplace=True)

print( df_expression.head() )


print( '***' )

# putting everything together



def expressionFilter(iterable , minExpr=10 , minSample=4):
    ''' returns True is at least <minSample> have a value of at least <minExpr> in <iterable>'''
    mask = iterable>=minExpr  # creating a mask to check for mininmum expression
    return sum(mask)>=minSample # Return True if at least minSample samples satisfy the filter


# apply the function along rows
expressionFilterMask =  df_expression.apply(expressionFilter , axis=1) 

print('expression filter for lowly expressed genes')
print( expressionFilterMask.value_counts() ) # we print the count of the custom mask we just created

# I use the filter to create a view of the DataFrame without the lowly expressed genes
df_expression_filtered = df_expression.loc[ expressionFilterMask , : ]

# Computing the mean of each column
column_means = df_expression_filtered.apply( np.mean , axis=0 ) # NB : df.mean(axis=0) works as well, try it out if you are curious

# Computing the standard deviation of each column
column_stds = df_expression_filtered.apply( np.std , axis=0 )

# centering and scaling -> removing the mean and dividing by the standard deviation
df_centered_scaled = ( df_expression_filtered - column_means ) / column_stds # NB : I put the result in a new DataFrame

print("Head of the filtered, centered, and scaled DataFrame")
print( df_centered_scaled.head() )




